<?php
/*
Plugin Name: Foliodock API
Plugin URI: http://wordpress.org/extend/plugins/foliodock-api/
Description: JSON API to build your own portfolio Webapp with your Wordpress Posts with the help of Foliodock.com.
Version: 1.0
Author: Timo Höner
Author URI: http://foliodock.com/
*/

$dir = foliodock_api_dir();
@include_once "$dir/singletons/api.php";
@include_once "$dir/singletons/query.php";
@include_once "$dir/singletons/introspector.php";
@include_once "$dir/singletons/response.php";
@include_once "$dir/models/post.php";
@include_once "$dir/models/comment.php";
@include_once "$dir/models/category.php";
@include_once "$dir/models/tag.php";
@include_once "$dir/models/author.php";
@include_once "$dir/models/attachment.php";

function foliodock_api_init() {
  global $foliodock_api;
  if (phpversion() < 5) {
    add_action('admin_notices', 'foliodock_api_php_version_warning');
    return;
  }
  if (!class_exists('Foliodock_API')) {
    add_action('admin_notices', 'foliodock_api_class_warning');
    return;
  }
  add_filter('rewrite_rules_array', 'foliodock_api_rewrites');
  $foliodock_api = new Foliodock_API();
}

function foliodock_api_php_version_warning() {
  echo "<div id=\"foliodock-api-warning\" class=\"updated fade\"><p>Sorry, Foliodock API requires PHP version 5.0 or greater.</p></div>";
}

function foliodock_api_class_warning() {
  echo "<div id=\"foliodock-api-warning\" class=\"updated fade\"><p>Oops, Foliodock_API class not found. If you've defined a Foliodock_API_DIR constant, double check that the path is correct.</p></div>";
}

function foliodock_api_activation() {
  // Add the rewrite rule on activation
  global $wp_rewrite;
  add_filter('rewrite_rules_array', 'foliodock_api_rewrites');
  $wp_rewrite->flush_rules();
}

function foliodock_api_deactivation() {
  // Remove the rewrite rule on deactivation
  global $wp_rewrite;
  $wp_rewrite->flush_rules();
}

function foliodock_api_rewrites($wp_rules) {
  $base = get_option('foliodock_api_base', 'api');
  if (empty($base)) {
    return $wp_rules;
  }
  $foliodock_api_rules = array(
    "$base\$" => 'index.php?json=info',
    "$base/(.+)\$" => 'index.php?json=$matches[1]'
  );
  return array_merge($foliodock_api_rules, $wp_rules);
}

function foliodock_api_dir() {
  if (defined('Foliodock_API_DIR') && file_exists(Foliodock_API_DIR)) {
    return Foliodock_API_DIR;
  } else {
    return dirname(__FILE__);
  }
}

// Add initialization and activation hooks
add_action('init', 'foliodock_api_init');
register_activation_hook("$dir/foliodock-api.php", 'foliodock_api_activation');
register_deactivation_hook("$dir/foliodock-api.php", 'foliodock_api_deactivation');

?>
