<?php

class Foliodock_API {
  
  function __construct() {
    $this->query = new Foliodock_API_Query();
    $this->introspector = new Foliodock_API_Introspector();
    $this->response = new Foliodock_API_Response();
    add_action('template_redirect', array(&$this, 'template_redirect'));
    add_action('admin_menu', array(&$this, 'admin_menu'));
    add_action('update_option_foliodock_api_base', array(&$this, 'flush_rewrite_rules'));
    add_action('pre_update_option_foliodock_api_controllers', array(&$this, 'update_controllers'));
    // Add stylesheet
    add_action('admin_head', 'admin_register_head');
    function admin_register_head() {
      $url = plugins_url('assets/css/style.css', dirname(__FILE__));
      echo "<link rel='stylesheet' href='$url' />\n";
    }
  }
  
  function template_redirect() {
    // Check to see if there's an appropriate API controller + method    
    $controller = strtolower($this->query->get_controller());
    $available_controllers = $this->get_controllers();
    $enabled_controllers = explode(',', get_option('foliodock_api_controllers', 'core'));
    $active_controllers = array_intersect($available_controllers, $enabled_controllers);
    
    if ($controller) {
      
      if (!in_array($controller, $active_controllers)) {
        $this->error("Unknown controller '$controller'.");
      }
      
      $controller_path = $this->controller_path($controller);
      if (file_exists($controller_path)) {
        require_once $controller_path;
      }
      $controller_class = $this->controller_class($controller);
      
      if (!class_exists($controller_class)) {
        $this->error("Unknown controller '$controller_class'.");
      }
      
      $this->controller = new $controller_class();
      $method = $this->query->get_method($controller);
      
      if ($method) {
        
        $this->response->setup();
        
        // Run action hooks for method
        do_action("foliodock_api-{$controller}-$method");
        
        // Error out if nothing is found
        if ($method == '404') {
          $this->error('Not found');
        }
        
        // Run the method
        $result = $this->controller->$method();
        
        // Handle the result
        $this->response->respond($result);
        
        // Done!
        exit;
      }
    }
  }
  
  function admin_menu() {
    add_options_page('Foliodock API Settings', 'Foliodock API', 'manage_options', 'foliodock-api', array(&$this, 'admin_options'));
    add_options_page('', '', 'manage_options', 'foliodock-api-ajax', array(&$this, 'admin_options_ajax'));
  }
  
  function admin_options_ajax() {
      echo '<div id="ajaxdata">';
      if (get_option('foliodock_api_search_kind') == 'category') {
        $url = get_bloginfo('url').'/?json=get_category_posts&count=100&slug='.get_option('foliodock_api_search_id');
        $json = file_get_contents($url);
      } else if (get_option('foliodock_api_search_kind') == 'tag') {
        $url = get_bloginfo('url').'/?json=get_tag_posts&count=100&slug='.get_option('foliodock_api_search_id');
        $json = file_get_contents($url);
      }
      $posts = json_decode($json);
      if($posts->status == 'error') {
        echo '<div class="error">Sorry, there was an error: '.$posts->error.'</div>';
      } else if ($posts->status == 'ok') {
        $html = 'Please enter the following url in your Foliodock Settings:';
        $html .= '<div class="apiurl">';
        $html .= '<code>'.$url.'</code><br />';
        $html .= '</div>';
        $html .= '<h2>The following posts will be shown:</h2>';
        echo $html;
        foreach($posts->posts as $post) {
          if(count($post->attachments) > 0) {
            $html = '<div class="post">';
            $attachment = $post->attachments[0];
              if(count($attachment->images) > 0) {
                $html .= '<img src="'.$attachment->images->thumbnail->url.'">';
              }
              $html .= '<strong>'.$post->title.'</strong>';
              $html .= '</div>';
            echo $html;
          }
        }
      } else {
        echo '<div class="error">Sorry, there was an error something went wrong...</div>';
      }
      echo '</div>';
    
  }
  
  function admin_options() {
    if (!current_user_can('manage_options'))  {
      wp_die( __('You do not have sufficient permissions to access this page.') );
    }

    $available_controllers = $this->get_controllers();
    $active_controllers = explode(',', get_option('foliodock_api_controllers', 'core'));
    
    if (count($active_controllers) == 1 && empty($active_controllers[0])) {
      $active_controllers = array();
    }
    
    if (!empty($_REQUEST['_wpnonce']) && wp_verify_nonce($_REQUEST['_wpnonce'], "update-options")) {
      if ((!empty($_REQUEST['action']) || !empty($_REQUEST['action2'])) &&
          (!empty($_REQUEST['controller']) || !empty($_REQUEST['controllers']))) {
        if (!empty($_REQUEST['action'])) {
          $action = $_REQUEST['action'];
        } else {
          $action = $_REQUEST['action2'];
        }
        
        if (!empty($_REQUEST['controllers'])) {
          $controllers = $_REQUEST['controllers'];
        } else {
          $controllers = array($_REQUEST['controller']);
        }
        
        foreach ($controllers as $controller) {
          if (in_array($controller, $available_controllers)) {
            if ($action == 'activate' && !in_array($controller, $active_controllers)) {
              $active_controllers[] = $controller;
            } else if ($action == 'deactivate') {
              $index = array_search($controller, $active_controllers);
              if ($index !== false) {
                unset($active_controllers[$index]);
              }
            }
          }
        }
        $this->save_option('foliodock_api_controllers', implode(',', $active_controllers));
		
      }
    	if ($_REQUEST['action'] == 'savetag' AND isset($_REQUEST['id'])){
        $this->save_option('foliodock_api_search_id', $_REQUEST['id']);
        $this->save_option('foliodock_api_search_kind', 'tag');
    	}

    	if ($_REQUEST['action'] == 'savecategory' AND isset($_REQUEST['id'])){
        $this->save_option('foliodock_api_search_id', $_REQUEST['id']);
        $this->save_option('foliodock_api_search_kind', 'category');
    	}
      if (isset($_REQUEST['foliodock_api_base'])) {
        $this->save_option('foliodock_api_base', $_REQUEST['foliodock_api_base']);
      }
    }
    
    ?>
<div class="wrap">
  <div id="icon-options-general" class="icon32"><br /></div>
  <h2>
    Foliodock API
    <?php
        
    foreach ($available_controllers as $controller) {
          
      $error = false;
      $active = in_array($controller, $active_controllers);
      $info = $this->controller_info($controller);
          
      if (is_string($info)) {
        $active = false;
        $error = true;
        $info = array(
          'name' => $controller,
          'description' => "<p><strong>Error</strong>: $info</p>",
          'methods' => array(),
          'url' => null
        );
      }
          
      ?>
            <?php
                
            if ($active) {
              echo '<a href="' . wp_nonce_url('options-general.php?page=foliodock-api&amp;action=deactivate&amp;controller=' . $controller, 'update-options') . '" title="' . __('Deactivate this controller') . '" class="button">' . __('Deactivate') . '</a>';
            } else if (!$error) {
              echo '<a href="' . wp_nonce_url('options-general.php?page=foliodock-api&amp;action=activate&amp;controller=' . $controller, 'update-options') . '" title="' . __('Activate this controller') . '" class="button">' . __('Activate') . '</a>';
            }
                  
            if ($info['url']) {
              echo ' | ';
              echo '<a href="' . $info['url'] . '" target="_blank">Docs</a></div>';
            }
                
            ?>
    <?php } ?>
  </h2>
  <?php if ($active) : ?>
    <p>
      With this plugin you can show your Wordpress Portfolio Posts in your Tablet App on <a href="http://foliodock.com">foliodock.com</a>.<br />Just choose a <em>category</em> or <em>tag</em> and enter the corresponding <em>API-Url</em> in your Foliodock-Settings.
    </p>
  <?php else : ?>
    <p class="error">
      Please activate the API!
    </p>
  <?php endif ?>
  
  <?php if ($active) : ?>
  	<h2>Choose your Source</h2>
  	<p>Specify the Category or Tags you wan't to show on Foliodock.</p>
    <p>
  	<form>
  		<select name="list" size="1">
  			<option value="#">Choose</option>
  			<option value="#">-- TAGS : --------</option>
  			<?php
  			$tags = get_tags();
  			foreach ($tags as $tag){
  				echo '<option value="'.wp_nonce_url('options-general.php?page=foliodock-api&amp;action=savetag&amp;id=' . $tag->slug, 'update-options').'">'.$tag->name.'</option>';
  			}
  			?>
  			<option value="#">-- CATEGORIES : --------</option>
  			<?php
  			$args=array(
  			  'orderby' => 'name',
  			  'order' => 'ASC'
  			  );
  			$categories = get_categories($args);
  			foreach ($categories as $category){
  				echo '<option value="'.wp_nonce_url('options-general.php?page=foliodock-api&amp;action=savecategory&amp;id=' . $category->slug, 'update-options').'">'.$category->name.'</option>';
  			}
  			?>
  		</select>
  		<input TYPE="button" VALUE="choose this" class="button" onClick="top.location.href=this.form.list.options[this.form.list.selectedIndex].value">
  	</form>
    </p>

	  <?php if(get_option('foliodock_api_search_id', FALSE)) : ?>
      <h2>Your Data <small>(<?php echo get_option('foliodock_api_search_kind')?> "<?php echo get_option('foliodock_api_search_id')?>")</small></h2>
      <div class="postlist">
        <div class="load">
          <img src="<?php echo plugins_url('assets/img/ajax-loader.gif', dirname(__FILE__))?>" /> Loading Data – please be patient...
        </div>
        <div style="clear:both">&nbsp;</div>
      </div>
    <script>
      jQuery(document).ready(function($) {
        $('.postlist').load('./<?php echo wp_nonce_url('options-general.php?page=foliodock-api-ajax') ?> #ajaxdata', function(){
          $('.postlist .loader').hide();
        });
      });
    </script>
    <?php endif ?>
  <?php endif ?>
</div>
<?php
  }


  
  function print_controller_actions($name = 'action') {
    ?>
    <div class="tablenav">
      <div class="alignleft actions">
        <select name="<?php echo $name; ?>">
          <option selected="selected" value="-1">Bulk Actions</option>
          <option value="activate">Activate</option>
          <option value="deactivate">Deactivate</option>
        </select>
        <input type="submit" class="button-secondary action" id="doaction" name="doaction" value="Apply">
      </div>
      <div class="clear"></div>
    </div>
    <div class="clear"></div>
    <?php
  }
  
  function get_method_url($controller, $method, $options = '') {
    $url = get_bloginfo('url');
    $base = get_option('foliodock_api_base', 'api');
    $permalink_structure = get_option('permalink_structure', '');
    if (!empty($options) && is_array($options)) {
      $args = array();
      foreach ($options as $key => $value) {
        $args[] = urlencode($key) . '=' . urlencode($value);
      }
      $args = implode('&', $args);
    } else {
      $args = $options;
    }
    if ($controller != 'core') {
      $method = "$controller/$method";
    }
    if (!empty($base) && !empty($permalink_structure)) {
      if (!empty($args)) {
        $args = "?$args";
      }
      return "$url/$base/$method/$args";
    } else {
      return "$url?json=$method&$args";
    }
  }
  
  function save_option($id, $value) {
    $option_exists = (get_option($id, null) !== null);
    if ($option_exists) {
      update_option($id, $value);
    } else {
      add_option($id, $value);
    }
  }
  
  function get_controllers() {
    $controllers = array();
    $dir = foliodock_api_dir();
    $dh = opendir("$dir/controllers");
    while ($file = readdir($dh)) {
      if (preg_match('/(.+)\.php$/', $file, $matches)) {
        $controllers[] = $matches[1];
      }
    }
    $controllers = apply_filters('foliodock_api_controllers', $controllers);
    return array_map('strtolower', $controllers);
  }
  
  function controller_is_active($controller) {
    if (defined('Foliodock_API_CONTROLLERS')) {
      $default = Foliodock_API_CONTROLLERS;
    } else {
      $default = 'core';
    }
    $active_controllers = explode(',', get_option('foliodock_api_controllers', $default));
    return (in_array($controller, $active_controllers));
  }
  
  function update_controllers($controllers) {
    if (is_array($controllers)) {
      return implode(',', $controllers);
    } else {
      return $controllers;
    }
  }
  
  function controller_info($controller) {
    $path = $this->controller_path($controller);
    $class = $this->controller_class($controller);
    $response = array(
      'name' => $controller,
      'description' => '(No description available)',
      'methods' => array()
    );
    if (file_exists($path)) {
      $source = file_get_contents($path);
      if (preg_match('/^\s*Controller name:(.+)$/im', $source, $matches)) {
        $response['name'] = trim($matches[1]);
      }
      if (preg_match('/^\s*Controller description:(.+)$/im', $source, $matches)) {
        $response['description'] = trim($matches[1]);
      }
      if (preg_match('/^\s*Controller URI:(.+)$/im', $source, $matches)) {
        $response['docs'] = trim($matches[1]);
      }
      if (!class_exists($class)) {
        require_once($path);
      }
      $response['methods'] = get_class_methods($class);
      return $response;
    } else if (is_admin()) {
      return "Cannot find controller class '$class' (filtered path: $path).";
    } else {
      $this->error("Unknown controller '$controller'.");
    }
    return $response;
  }
  
  function controller_class($controller) {
    return "foliodock_api_{$controller}_controller";
  }
  
  function controller_path($controller) {
    $dir = foliodock_api_dir();
    $controller_class = $this->controller_class($controller);
    return apply_filters("{$controller_class}_path", "$dir/controllers/$controller.php");
  }
  
  function get_nonce_id($controller, $method) {
    $controller = strtolower($controller);
    $method = strtolower($method);
    return "foliodock_api-$controller-$method";
  }
  
  function flush_rewrite_rules() {
    global $wp_rewrite;
    $wp_rewrite->flush_rules();
  }
  
  function error($message = 'Unknown error', $status = 'error') {
    $this->response->respond(array(
      'error' => $message
    ), $status);
  }
  
  function include_value($key) {
    return $this->response->is_value_included($key);
  }
  
}

?>
